namespace Refactory {
	using System;
	using System.Text;
	
	public class LargeInteger : IComparable {
		
		
		public LargeInteger(int number) : this((number < 0 ? (uint)-number : (uint)number)) {
			if (number < 0) negative = true;
		}
		
		public LargeInteger(uint number) {
			ints = new uint[1];
			ints[0] = number;
			size = 1;
			negative = false;
			ReduceSize();
		}
		
		public LargeInteger(long number) : this((number < 0 ? (ulong)-number : (ulong)number)) {
			if (number < 0) negative = true;
		}
		
		public LargeInteger(ulong number) {
			size = 2;
			negative = false;
			ints = new uint[2];
			ints[0] = LowWord(number);
			ints[1] = (uint)(number >> WordSize);
			ReduceSize();
		}
		
		public LargeInteger(float number) : this((double)number) {}
		
		public LargeInteger(double number) {
			if (Double.IsInfinity(number)) throw new OverflowException("Cannot represent Infinity as a LargeInteger");
			if (Double.IsNaN(number)) throw new ArgumentException("Argument is NaN");
			
			if (number == 0.0) {
				size = 0;
				negative = false;
				return;
			}
			
			int shiftBits = 0;
			while (Math.Abs(number) > (double)(1UL << 63)) {
				number /= 2.0;
				shiftBits++;
			}
			
			ulong value = (ulong)Math.Abs(number);
			
			int low = shiftBits / WordSize;
			shiftBits %= WordSize;
			size = 2 + (shiftBits > 0 ? 1 : 0) + low;
			ints = new uint[size];
			
			ints[low] = LowWord(value << shiftBits);
			ints[low+1] = LowWord((value << shiftBits) >> WordSize);
			if (shiftBits > 0) ints[low+2] = LowWord(value >> ((WordSize * 2) - shiftBits));
			
			negative = (number < 0);
			
			ReduceSize();
		}
		
		public LargeInteger(uint[] digits, bool isNonPositive) {
			ints = (uint[])digits.Clone();
			size = digits.Length;
			ReduceSize();
			negative = isNonPositive && size > 0;
		}
		
		public LargeInteger(byte[] digits, bool isNonPositive) {
			ints = new uint[digits.Length / 4 + 1];
			size = ints.Length;
			
			int shiftBits = 0;
			int dest = 0;
			for (int i = 0; i < digits.Length; i++) {
				ints[dest] |= ((uint)digits[i] << shiftBits);
				if (i % 4 == 3) {
					shiftBits = 0;
					dest++;
				} else {
					shiftBits += 8;
				}
			}
			ReduceSize();
			negative = isNonPositive && size > 0;
		}
		
		public LargeInteger(string number) : this(number,10) {}
		
		public LargeInteger(string number, int radix) {
			if (radix < 2 || radix > 36) throw new ArgumentException("Invalid radix");
			if (number.Length == 0) throw new ArgumentException("Empty string");
			
			LargeInteger r = new LargeInteger(radix);
			LargeInteger n = Zero;
			
			int i = 0;
			if (number[i] == '-') {
				negative = true;
				i++;
			}
			
			for (;i<number.Length;i++) {
				int digitValue = RadixDigits.IndexOf(number[i]);
				if (digitValue < 0 || digitValue >= radix) throw new ArgumentException("Invalid number");
				n = (n * r) + (new LargeInteger(digitValue));
			}
			
			ints = n.ints;
			size = n.size;
		}
		
		public LargeInteger(Decimal number) {
			int[] bits = Decimal.GetBits(number);
			LargeInteger temp = new LargeInteger();
			
			temp.ints = new uint[3];
			temp.size = 3;
			temp.negative = false;
			for (int i = 0; i < 3; i++) temp.ints[i] = (uint)bits[i];
			temp.ReduceSize();
			
			uint scale = (uint)((bits[3] >> 16) & 0xF);
			temp /= (new LargeInteger(10)).Pow(scale);
			ints = temp.ints;
			size = temp.size;
			
			negative = (bits[3] < 0) && (size > 0);
		}
		
		public static LargeInteger operator -(LargeInteger l) {
			return l.Negated();
		}
		
		public static LargeInteger operator +(LargeInteger l1, LargeInteger l2) {
			return l1.Add(l2);
		}
		
		public static LargeInteger operator -(LargeInteger l1, LargeInteger l2) {
			return l1.Subtract(l2);
		}
		
		public static LargeInteger operator *(LargeInteger l1, LargeInteger l2) {
			return l1.Multiply(l2);
		}
		
		public static LargeInteger operator /(LargeInteger n, LargeInteger d) {
			return n.Divide(d);
		}
		
		public static LargeInteger operator %(LargeInteger n, LargeInteger d) {
			return n.QuotientAndRemainder(d)[1];
		}
		
		public static LargeInteger operator <<(LargeInteger l, int s) {
			return l.BitShift(s);
		}
		
		public static LargeInteger operator >>(LargeInteger l, int s) {
			return l.BitShift(-s);
		}
		
		public static bool operator ==(LargeInteger l1, LargeInteger l2) {
			return (l1.CompareTo(l2) == 0);
		}
		
		public static bool operator !=(LargeInteger l1, LargeInteger l2) {
			return (l1.CompareTo(l2) != 0);
		}
		
		public static bool operator <=(LargeInteger l1, LargeInteger l2) {
			return (l1.CompareTo(l2) <= 0);
		}
		
		public static bool operator <(LargeInteger l1, LargeInteger l2) {
			return (l1.CompareTo(l2) < 0);
		}
		
		public static bool operator >=(LargeInteger l1, LargeInteger l2) {
			return (l1.CompareTo(l2) >= 0);
		}
		
		public static bool operator >(LargeInteger l1, LargeInteger l2) {
			return (l1.CompareTo(l2) > 0);
		}
		
		public static LargeInteger operator &(LargeInteger l1, LargeInteger l2) {
			return l1.BitAnd(l2);
		}
		
		public static LargeInteger operator |(LargeInteger l1, LargeInteger l2) {
			return l1.BitOr(l2);
		}
		
		public static LargeInteger operator ^(LargeInteger l1, LargeInteger l2) {
			return l1.BitXor(l2);
		}
		
		public override bool Equals(object o) {
			if (this.GetType() != o.GetType()) return false;
			return this == (LargeInteger)o;
		}
		
		public override int GetHashCode() {
			int hash = 0;
			
			for (int i = 0; i < size; i++) {
				unchecked { hash ^= (int)ints[i]; }
			}
			
			return hash;
		}
		
		public int CompareTo(object o) {
			return CompareTo((LargeInteger)o);
		}
		
		public int CompareTo(LargeInteger l) {
			if (negative != l.negative) return negative ? -1 : 1;
			
			return AbsCompareTo(l);
		}
		
		public int AbsCompareTo(LargeInteger l) {
			if (size != l.size) return size - l.size;
			
			for (int i = size - 1;i >= 0;i--) {
				if (ints[i] < l.ints[i]) return -1;
				if (ints[i] > l.ints[i]) return 1;
			}
			return 0;
		}
		
		public override string ToString() {
			return ToString(10);
		}
		
		public bool NonPositive {
			get { return negative || size == 0; }
		}
		
		public bool Negative {
			get { return negative; }
		}
		
		public bool NonNegative {
			get { return !negative; }
		}
		
		public bool Positive {
			get { return !negative && size > 0; }
		}
		
		public LargeInteger Negated() {
			LargeInteger answer = new LargeInteger();
			answer.size = size;
			if (size > 0) {
				answer.ints = ints;
				answer.negative = !negative;
			}
			return answer;
		}
		
		public LargeInteger Add(LargeInteger l) {
			if (size == 0) return l;
			if (l.size == 0) return this;
			
			if (negative == l.negative) {
				return BasicAdd(l);
			} else {
				return BasicSubtract(l.Negated());
			}
		}
		
		public LargeInteger Subtract(LargeInteger l) {
			if (size == 0) return l.Negated();
			if (l.size == 0) return this;
			
			if (negative != l.negative) {
				return BasicAdd(l);
			} else {		
				return BasicSubtract(l);
			}
		}
		
		public LargeInteger SubtractedFrom(LargeInteger l) {
			return l.Subtract(this);
		}
		
		public LargeInteger Multiply(LargeInteger l) {
			if (size == 0) return this;
			if (l.size == 0) return l;
			
			LargeInteger answer = new LargeInteger();
			
			answer.size = size + l.size;
			answer.ints = new uint[answer.size];
			
			for (int i=0;i<size;i++) {
				ulong valueI = ints[i];
				if (valueI > 0) {
					int dest = i;
					for (int j=0;j<l.size;j++,dest++) {
						ulong valueJ = l.ints[j];
						if (valueJ > 0) {
							answer.AddValueAt(valueI * valueJ,dest);
						}
					}
				}
			}
			
			answer.ReduceSize();
			answer.negative = negative ^ l.negative;
			return answer;
		}
		
		protected void AddValueAt(ulong value, int index) {
			ulong carry = value >> WordSize;
			int dest = index;
			ulong newvalue = (value & UInt32.MaxValue) + (ulong)ints[dest];
			ints[dest] = LowWord(newvalue);
			
			carry += newvalue >> WordSize;
			
			while (carry != 0) {
				dest++;
				carry += (ulong)ints[dest];
				ints[dest] = LowWord(carry);
				carry >>= WordSize;
			}
			
			return;
		}
		
		public LargeInteger Divide(LargeInteger l) {
			return QuotientAndRemainder(l)[0];
		}
		
		public LargeInteger DividedBy(LargeInteger l) {
			return l.Divide(this);
		}
		
		public LargeInteger[] QuotientAndRemainder(LargeInteger l) {
			LargeInteger[] answer;
			
			if (l.size <= 1) {
				answer = IntegerQuotientAndRemainder(l.ToUInt32());
				if (l.negative) {
					answer[0].negative = !answer[0].negative;
				}
				return answer;
			}
			
			answer = new LargeInteger[2];
			
			int comp = AbsCompareTo(l);
			
			if (comp == 0) {
				answer[0] = new LargeInteger(1);
				if (negative ^ l.negative) answer[0].negative = true;
				answer[1] = Zero;
				return answer;
			}
			
			if (comp < 0) {
				answer[0] = Zero;
				answer[1] = new LargeInteger();
				answer[1].ints = ints;
				answer[1].size = size;
				answer[1].negative = negative ^ l.negative;
				return answer;
			}
			
			LargeInteger quotient = new LargeInteger();
			
			int shiftBits = 0;
			
			// D1 -- Normalize
			for (uint i = l.ints[l.size-1]; i < HighWordBit; i <<= 1) shiftBits++;
			
			LargeInteger u = this << shiftBits;
			LargeInteger v = l << shiftBits;
			
			int n = v.size;
			int m = u.size - v.size;
			const ulong b = 0x100000000;
			
			quotient.size = m + 1;
			quotient.ints = new uint[quotient.size];
			
			ulong mostSignificantDenominatorDigit = (ulong)v.ints[n-1];
			ulong secondMostSignificantDenominatorDigit = (ulong)v.ints[n-2];
			
			for (int j = m;j>=0;j--) {
				// D3 -- Calculate quotient digit
				ulong topTwoNumeratorDigits = (((ulong)u.DigitAt(j + n) << WordSize) | (ulong)u.DigitAt(j + n - 1));
				ulong quotientDigit = topTwoNumeratorDigits / mostSignificantDenominatorDigit;
				ulong remainderDigit = topTwoNumeratorDigits % mostSignificantDenominatorDigit;
				bool repeat;
				do {
					repeat = quotientDigit == b || 
						(quotientDigit * secondMostSignificantDenominatorDigit > 
							((remainderDigit << WordSize) |  (ulong)u.DigitAt(j + n - 2)));
					if (repeat) {
						quotientDigit--;
						remainderDigit += mostSignificantDenominatorDigit;
					}
				} while (repeat && (remainderDigit < b)); // This should only repeat once due to step D1 -- Normalize
				
				// D4 -- Multiply & subtract
				if (quotientDigit > 0) {
					int shiftDistance = WordSize * j;
					bool isNonPositive;
					
					isNonPositive = u.SubtractFromSelf(v * (new LargeInteger(quotientDigit)),j);
					
					// D5, D6 -- Test remainder & Add back if true
					if (isNonPositive) {
						quotientDigit--;
						u.AddBack(v,j);
					}
					u.ReduceSize();
				}
				
				quotient.ints[j] = LowWord(quotientDigit);
			}
			
			// D8 -- Unnormalize
			quotient.ReduceSize();
			answer[0] = quotient;
		    answer[1] = u >> shiftBits;
		    
		    if (answer[0].size > 0) answer[0].negative = (negative != l.negative);
			
			return answer;
		}
		
		protected void AddBack(LargeInteger l, int position) {
			ulong carry = 0;
			int i = 0;
			for (;i < l.size && position < size;i++,position++) {
				carry += (ulong)ints[position] + (ulong)l.ints[i];
				ints[position] = LowWord(carry);
				carry >>= WordSize;
			}
			
			for (;position < size && carry != 0;position++) {
				carry += (ulong)ints[position];
				ints[position] = LowWord(carry);
				carry >>= WordSize;
			}
		}
		
		protected bool SubtractFromSelf(LargeInteger l, int position) {
			long carry = 0;
			int i = 0;
			
			for (;i<l.size && position < size;i++,position++) {
				carry += (long)ints[position] - (long)l.ints[i];
				ints[position] = LowWord(carry);
				carry >>= WordSize;
			}
			
			for (;position < size && (carry != 0);position++) {
				carry += (long)ints[i];
				ints[position] = LowWord(carry);
				carry >>= WordSize;
			}
			
			return carry != 0;
		}
		
		private LargeInteger[] IntegerQuotientAndRemainder(uint div) {
			if (div == 0) throw new DivideByZeroException();
			
			LargeInteger answer = new LargeInteger();
			LargeInteger[] retval = new LargeInteger[2];
			
			if (div == 1) {
				retval[0] = this;
				retval[1] = Zero;
				return retval;
			}
			
			ulong carry = 0;
			
			answer.size = size;
			answer.ints = new uint[size];
			
			for (int j = size-1;j>=0;j--) {
				carry |= ints[j];
				answer.ints[j] = LowWord(carry / div);
				carry = (carry % div) << WordSize;
			}
			
			answer.ReduceSize();
			answer.negative = answer.size > 0 && negative;
			long result = (long)(carry >> WordSize);
			if (negative) result = -result;
			
			retval[0] = answer;
			retval[1] = new LargeInteger(result);
			
			return retval;
		}
		
		public static LargeInteger Factorial(uint i) {
			if (i < 1) throw new ArgumentException("Cannot compute factorial for numbers <= 0");
			LargeInteger answer = One;
			ulong max = UInt64.MaxValue / (ulong)i;
			ulong temp = 1;
			
			for (uint current = i;current>1;current--) {
				if (temp <= max) {
					temp *= (ulong)current;
				} else {
					answer *= new LargeInteger(temp);
					temp = (ulong)current;
				}
			}
			return answer * new LargeInteger(temp);
		}
		
		public LargeInteger BitShift(int i) {
			LargeInteger answer = new LargeInteger();
			int bits = i % WordSize;
			int words = i / WordSize;
			ulong carry = 0;
			int dest, origin;
			
			answer.size = size + words + (bits > 0 ? 1 : 0);
		    if (answer.size <= 0) {
		        answer.size = 0;
				return answer;
			}
			answer.ints = new uint[answer.size];
			
			if (i > 0) {
				for (dest = words, origin = 0; origin<size; dest++,origin++) {
					carry |= (((ulong)ints[origin]) << bits);
					answer.ints[dest] = LowWord(carry);
					carry >>= WordSize;
				}
				if (bits > 0) answer.ints[answer.size-1] = (uint)carry;
			} else {
				bits = -bits;
				for (dest = size-1+words, origin = size - 1; dest >= 0; dest--,origin--) {
					carry |= (ulong)ints[origin];
					answer.ints[dest] = LowWord(carry >> bits);
					carry <<= WordSize;
				}
			}
			
			answer.ReduceSize();
			if (answer.size > 0) answer.negative = negative;
			return answer;
		}
		
		public LargeInteger BitAnd(LargeInteger l) {
			LargeInteger answer = new LargeInteger();
			
			answer.size = size > l.size ? l.size : size;
			answer.ints = new uint[answer.size];
			
			for (int i = 0; i < answer.size; i++) answer.ints[i] = ints[i] & l.ints[i];
			
			answer.ReduceSize();
			return answer;
		}
		
		public LargeInteger BitOr(LargeInteger l) {
			LargeInteger answer = new LargeInteger();
			LargeInteger larger = size < l.size ? l : this;
			int minSize = size < l.size ? size : l.size;
			int maxSize = size > l.size ? size : l.size;
			
			answer.size = maxSize;
			answer.ints = new uint[maxSize];
			
			int i = 0;
			for (; i < minSize; i++) answer.ints[i] = ints[i] | l.ints[i];
			for (; i < maxSize; i++) answer.ints[i] = larger.ints[i];
			
			return answer;
		}
		
		public LargeInteger BitXor(LargeInteger l) {
			LargeInteger answer = new LargeInteger();
			LargeInteger larger = size < l.size ? l : this;
			int minSize = size < l.size ? size : l.size;
			int maxSize = size > l.size ? size : l.size;
			
			answer.size = maxSize;
			answer.ints = new uint[maxSize];
			
			int i = 0;
			for (; i < minSize; i++) answer.ints[i] = ints[i] ^ l.ints[i];
			for (; i < maxSize; i++) answer.ints[i] = larger.ints[i];
			
			answer.ReduceSize();
			return answer;
		}
		
		public float ToFloat() {
			return (float)ToDouble();
		}
		
		public double ToDouble() {
			if (size == 0) return 0.0;
			if (size == 1) return (negative ? -(double)ints[0] : (double)ints[0]);
			
			ulong value = ((ulong)(ints[size-1])<<WordSize) + (ulong)ints[size-2];
			
			if (size == 2) return (negative ? -(double)value : (double)value);
			
			int shift = 0;
			while (value <= 0x1FFFFFFFFFFFFF) { // get 53-bits for the double
				shift++; 
				value <<= 1;
			}
			
			if (shift != 0) value |= (ints[size - 3] >> (64 - shift));
			
			double retval = (double)value;
			if (size > 2) {
				retval *= Math.Pow(2.0,((size - 2) * WordSize) - shift);
			}
			
			if (negative) retval = -retval;
			
			return retval;
		}
		
		public long ToInt64() {
			ulong value = ToUInt64();
		    
		    if (value > 0x8000000000000000UL) throw new OverflowException();
		    
		    if (value == 0x8000000000000000UL) {
		        if (negative) {
		            return Int64.MinValue;
		        } else {
		            throw new OverflowException();
		        }
		    }
			return (negative ? -(long)value : (long)value);
		}
		
		public ulong ToUInt64() {
			if (size > 2) throw new OverflowException();
			
			if (size == 0) return 0;
			if (size == 1) return ints[0];
			
			return (((ulong)ints[1]) << WordSize) | ((ulong)ints[0]);
		}
		
		public int ToInt32() {
			if (size > 1) throw new OverflowException();
			
			uint value = ToUInt32();
		    
		    if (value > 0x80000000) throw new OverflowException();
		    
		    if (value == 0x80000000) {
		        if (negative) {
		            return Int32.MinValue;
		        } else {
		            throw new OverflowException();
		        }
		    }
			return (negative ? -(int)value : (int)value);
		}
		
		public uint ToUInt32() {
			if (size > 1) throw new OverflowException();
			
			if (size == 0) return 0;
			
			return ints[0];
		}
		
		public Decimal ToDecimal() {
			return new Decimal(size >= 1 ? (int)ints[0] : 0,
			                   size >= 2 ? (int)ints[1] : 0,
			                   size >= 3 ? (int)ints[2] : 0,
			                   negative,
			                   0);
		}
		
		public bool IsInt64() {
			if (size <= 1) return true;
			if (size > 2) return false;
			if (negative) {
				if (ints[1] == HighWordBit) {
					return ints[0] == 0;
				} else {
					return ints[1] <= HighWordBit;
				}
			} else {
				return ints[1] < HighWordBit;
			}
		}
		
		public bool IsUInt64() {
			return size <= 2;
		}
		
		public bool IsInt32() {
		    if (size == 0) return true;
			if (size > 1) return false;
			if (negative) {
				return ints[0] <= HighWordBit;
			} else {
				return ints[0] < HighWordBit;
			}
		}
		
		public bool IsUInt32() {
			return size <= 1;
		}
		
		public bool IsDecimal() {
			return size <= 3;
		}
		
		protected uint LowWord(ulong i) {
			return (uint)(i & 0xFFFFFFFF);
		}
		
		protected uint LowWord(long i) {
			return (uint)(i & 0xFFFFFFFF);
		}
		
		protected uint DigitAt(int i) {
			return (size > i && i >= 0) ? ints[i] : 0U;
		}
		
		protected void ReduceSize() {
			for (int i=size-1;i>=0;i--) {
				if (ints[i] != 0) {
					size = i + 1;
					return;
				}
			}
			size = 0;
		}
		
		protected LargeInteger BasicAdd(LargeInteger l) {
			if (size == 0) return l;
			if (l.size == 0) return this;
			
			LargeInteger answer = new LargeInteger();
			LargeInteger minInt, maxInt;
			
			if (size < l.size) {
				minInt = this;
				maxInt = l;
			} else {
				minInt = l;
				maxInt = this;
			}
			
			answer.size = maxInt.size + 1;
			answer.ints = new uint[answer.size];
			ulong carry = 0;
			
			int i = 0;
			for (;i < minInt.size; i++) {
				carry += (ulong)minInt.ints[i] + (ulong)maxInt.ints[i];
				answer.ints[i] = LowWord(carry);
				carry >>= WordSize;
			}
			
			for (;i < maxInt.size; i++) {
				carry += (ulong)maxInt.ints[i];
				answer.ints[i] = LowWord(carry);
				carry >>= WordSize;
			}
			
			answer.ints[answer.size-1] = (uint)carry;
			
			answer.ReduceSize();
			answer.negative = negative;
			return answer;
		}
		
		protected LargeInteger BasicSubtract(LargeInteger l) {
			LargeInteger answer = new LargeInteger();
			LargeInteger maxInt, minInt;
			
			int compare = AbsCompareTo(l);
			if (compare == 0) {
				answer.size = 0;
				return answer;
			}
			
			if (compare < 0) {
				maxInt = l;
				minInt = this;
				answer.negative = !l.negative;
			} else {
				maxInt = this;
				minInt = l;
				answer.negative = negative;
			}
			
			long carry = 0;
			
			answer.ints = new uint[maxInt.size];
			answer.size = maxInt.size;
			
			int i = 0;
			for (;i<minInt.size; i++) {
				carry += (long)maxInt.ints[i] - (long)minInt.ints[i];
				answer.ints[i] = LowWord(carry);
				carry >>= WordSize;
			}
			
			for (;i<maxInt.size; i++) {
				carry += (long)maxInt.ints[i];
				answer.ints[i] = LowWord(carry);
				carry >>= WordSize;
			}
			
			answer.ReduceSize();
			return answer;
		}
		
		public string ToString(int radix) {
			StringBuilder s = new StringBuilder();
			LargeInteger current = this;
			
			if (negative) {
				current = Negated();
			}
			
			do {
				LargeInteger[] quotientAndRemainder = current.IntegerQuotientAndRemainder((uint)radix);
				s.Append(RadixDigits[quotientAndRemainder[1].ToInt32()]);
				current = quotientAndRemainder[0];
			} while (current.size > 0);
			
			
			StringBuilder result = new StringBuilder(s.Length + (Negative ? 1 : 0));
			if (Negative) result.Append('-');
			
			for (int i = s.Length - 1; i >= 0; i--) result.Append(s[i]);
			
			return result.ToString();
		}
		
		public LargeInteger Pow(uint power) {			
			LargeInteger result = One;
			
			for (int i = 0; i < 32; i++) {
				result *= result;
				if ((power & HighWordBit) == HighWordBit) result *= this;
				unchecked { power <<= 1; }
			}
			
			return result;
		}
		
		public LargeInteger PowMod(LargeInteger power, LargeInteger mod) {
			LargeInteger result = One;
			
			for (int j = power.size - 1; j >= 0; j--) {
				uint intPower = power.ints[j];
				for (int i = 0; i < WordSize; i++) {
					result *= result;
					result %= mod;
					if ((intPower & HighWordBit) == HighWordBit) {
						result *= this;
						result %= mod;
					}
					unchecked { intPower <<= 1; }
				}
			}
			
			return result;
		}
		
		protected static LargeInteger one;
		protected static LargeInteger zero;
		
		static LargeInteger() {
			one = new LargeInteger(1);
			zero = new LargeInteger(0);
		}
		
		public static LargeInteger One {
			get { return one; }
		}
		
		public static LargeInteger Zero {
			get { return zero; }
		}
		
		public int BitCount {
			get {
				if (size == 0) return 0;
				int count = (size - 1) * WordSize;
				for (uint i = ints[size - 1]; i > 0; i >>= 1) count++;
				return count;
			}
		}
		
		public bool this[int i] {
			get {
				int offset = i / WordSize;
				if (offset > size - 1) return false;
				
				uint bit = 1U << (i % WordSize);
				
				return (ints[offset] & bit) == bit;				
			}
		}
		
		public LargeInteger Abs() {
			if (negative) {
				return -this;
			} else {
				return this;
			}
		}
		
		public LargeInteger Gcd(LargeInteger l) {
			LargeInteger max, min;
			
			if (this.AbsCompareTo(l) >= 0) {
				max = Abs();
				min = l.Abs();
			} else {
				max = l.Abs();
				min = Abs();
			}
			
			while (min != Zero) {
				LargeInteger temp = max % min;
				max = min;
				min = temp;
			}
			
			return max;
		}
		
		protected uint[] ints;
		protected int size;
		protected bool negative;
		
		protected const string RadixDigits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		protected const int WordSize = 32;
		protected const uint HighWordBit = 0x80000000U;
		
		protected LargeInteger() {
			size = 0;
			negative = false;
		}
	
	}
}
